@@ -13,15 +13,17 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
-from oslo.config import cfg
+from oslo.serialization import jsonutils
+from oslo_config import cfg
+from oslo_log import log as logging
 import webob.exc
 
 from glance.api import policy
 from glance.common import wsgi
 import glance.context
-from glance.openstack.common import jsonutils
-import glance.openstack.common.log as logging
+from glance import i18n
 
+_ = i18n._
 
 context_opts = [
     cfg.BoolOpt('owner_is_tenant', default=True,
@@ -51,7 +53,14 @@ class BaseContextMiddleware(wsgi.Middleware):
         except AttributeError:
             LOG.warn(_('Unable to retrieve request id from context'))
         else:
-            resp.headers['x-openstack-request-id'] = 'req-%s' % request_id
+            # For python 3 compatibility need to use bytes type
+            prefix = b'req-' if isinstance(request_id, bytes) else 'req-'
+
+            if not request_id.startswith(prefix):
+                request_id = prefix + request_id
+
+            resp.headers['x-openstack-request-id'] = request_id
+
         return resp
 
 
@@ -91,12 +100,12 @@ class ContextMiddleware(BaseContextMiddleware):
         return glance.context.RequestContext(**kwargs)
 
     def _get_authenticated_context(self, req):
-        #NOTE(bcwaldon): X-Roles is a csv string, but we need to parse
+        # NOTE(bcwaldon): X-Roles is a csv string, but we need to parse
         # it into a list to be useful
         roles_header = req.headers.get('X-Roles', '')
         roles = [r.strip().lower() for r in roles_header.split(',')]
 
-        #NOTE(bcwaldon): This header is deprecated in favor of X-Auth-Token
+        # NOTE(bcwaldon): This header is deprecated in favor of X-Auth-Token
         deprecated_token = req.headers.get('X-Storage-Token')
 
         service_catalog = None
