@@ -3,6 +3,7 @@
 # Copyright 2010-2011 OpenStack Foundation
 # Copyright 2012 Justin Santa Barbara
 # Copyright 2013 IBM Corp.
+# Copyright 2015 Mirantis, Inc.
 # All Rights Reserved.
 #
 #    Licensed under the Apache License, Version 2.0 (the "License"); you may
@@ -22,23 +23,24 @@
 
 import threading
 
-from oslo.config import cfg
-from oslo.db import exception as db_exception
-from oslo.db.sqlalchemy import session
+from oslo_config import cfg
+from oslo_db import exception as db_exception
+from oslo_db.sqlalchemy import session
+from oslo_log import log as logging
+from oslo_utils import timeutils
 import osprofiler.sqlalchemy
 from retrying import retry
 import six
-from six.moves import xrange
+# NOTE(jokke): simplified transition to py3, behaves like py2 xrange
+from six.moves import range
 import sqlalchemy
 import sqlalchemy.orm as sa_orm
 import sqlalchemy.sql as sa_sql
 
+from glance import artifacts as ga
 from glance.common import exception
-from glance.db.sqlalchemy import models
-from glance import i18n
-import glance.openstack.common.log as os_logging
-from glance.openstack.common import timeutils
-
+from glance.common import utils
+from glance.db.sqlalchemy import artifacts
 from glance.db.sqlalchemy.metadef_api import namespace as metadef_namespace_api
 from glance.db.sqlalchemy.metadef_api import object as metadef_object_api
 from glance.db.sqlalchemy.metadef_api import property as metadef_property_api
@@ -46,19 +48,22 @@ from glance.db.sqlalchemy.metadef_api\
     import resource_type as metadef_resource_type_api
 from glance.db.sqlalchemy.metadef_api\
     import resource_type_association as metadef_association_api
+from glance.db.sqlalchemy.metadef_api import tag as metadef_tag_api
+from glance.db.sqlalchemy import models
+from glance import i18n
 
 BASE = models.BASE
 sa_logger = None
-LOG = os_logging.getLogger(__name__)
+LOG = logging.getLogger(__name__)
+_ = i18n._
 _LI = i18n._LI
 _LW = i18n._LW
 
 
 STATUSES = ['active', 'saving', 'queued', 'killed', 'pending_delete',
-            'deleted']
+            'deleted', 'deactivated']
 
 CONF = cfg.CONF
-CONF.import_opt('debug', 'glance.openstack.common.log')
 CONF.import_group("profiler", "glance.common.wsgi")
 
 _FACADE = None
@@ -109,7 +114,7 @@ def clear_db_env():
 
 def _check_mutate_authorization(context, image_ref):
     if not is_image_mutable(context, image_ref):
-        LOG.info(_LI("Attempted to modify image user did not own."))
+        LOG.warn(_LW("Attempted to modify image user did not own."))
         msg = _("You do not own this image")
         if image_ref.is_public:
             exc_class = exception.ForbiddenPublicImage
@@ -157,10 +162,10 @@ def image_destroy(context, image_id):
 
         _image_tag_delete_all(context, image_id, delete_time, session)
 
-    return _normalize_locations(image_ref)
+    return _normalize_locations(context, image_ref)
 
 
-def _normalize_locations(image, force_show_deleted=False):
+def _normalize_locations(context, image, force_show_deleted=False):
     """
     Generate suitable dictionary list for locations field of image.
 
@@ -168,6 +173,11 @@ def _normalize_locations(image, force_show_deleted=False):
     from image query.
     """
 
+    if image['status'] == 'deactivated' and not context.is_admin:
+        # Locations are not returned for a deactivated image for non-admin user
+        image['locations'] = []
+        return image
+
     if force_show_deleted:
         locations = image['locations']
     else:
@@ -189,7 +199,7 @@ def _normalize_tags(image):
 def image_get(context, image_id, session=None, force_show_deleted=False):
     image = _image_get(context, image_id, session=session,
                        force_show_deleted=force_show_deleted)
-    image = _normalize_locations(image.to_dict(),
+    image = _normalize_locations(context, image.to_dict(),
                                  force_show_deleted=force_show_deleted)
     return image
 
@@ -203,8 +213,8 @@ def _check_image_id(image_id):
     :param image_id: The id of the image we want to check
     :return: Raise NoFound exception if given image id is invalid
     """
-    if image_id and \
-            len(image_id) > models.Image.id.property.columns[0].type.length:
+    if (image_id and
+       len(image_id) > models.Image.id.property.columns[0].type.length):
         raise exception.NotFound()
 
 
@@ -214,13 +224,13 @@ def _image_get(context, image_id, session=None, force_show_deleted=False):
     session = session or get_session()
 
     try:
-        query = session.query(models.Image)\
-                       .options(sa_orm.joinedload(models.Image.properties))\
-                       .options(sa_orm.joinedload(models.Image.locations))\
-                       .filter_by(id=image_id)
+        query = session.query(models.Image).options(
+            sa_orm.joinedload(models.Image.properties)).options(
+                sa_orm.joinedload(
+                    models.Image.locations)).filter_by(id=image_id)
 
         # filter out deleted images if context disallows it
-        if not force_show_deleted and not _can_show_deleted(context):
+        if not force_show_deleted and not context.can_see_deleted:
             query = query.filter_by(deleted=False)
 
         image = query.one()
@@ -359,9 +369,9 @@ def _paginate_query(query, model, limit, sort_keys, marker=None,
 
         # Build up an array of sort criteria as in the docstring
         criteria_list = []
-        for i in xrange(len(sort_keys)):
+        for i in range(len(sort_keys)):
             crit_attrs = []
-            for j in xrange(i):
+            for j in range(i):
                 model_attr = getattr(model, sort_keys[j])
                 default = None if isinstance(
                     model_attr.property.columns[0].type,
@@ -398,7 +408,8 @@ def _paginate_query(query, model, limit, sort_keys, marker=None,
 
 
 def _make_conditions_from_filters(filters, is_public=None):
-    #NOTE(venkatesh) make copy of the filters are to be altered in this method.
+    # NOTE(venkatesh) make copy of the filters are to be altered in this
+    # method.
     filters = filters.copy()
 
     image_conditions = []
@@ -487,9 +498,8 @@ def _select_images_query(context, image_conditions, admin_as_user,
 
     regular_user = (not context.is_admin) or admin_as_user
 
-    query_member = session.query(models.Image) \
-        .join(models.Image.members) \
-        .filter(img_conditional_clause)
+    query_member = session.query(models.Image).join(
+        models.Image.members).filter(img_conditional_clause)
     if regular_user:
         member_filters = [models.ImageMember.deleted == False]
         if context.owner is not None:
@@ -499,32 +509,31 @@ def _select_images_query(context, image_conditions, admin_as_user,
                     models.ImageMember.status == member_status])
         query_member = query_member.filter(sa_sql.and_(*member_filters))
 
-    #NOTE(venkatesh) if the 'visibility' is set to 'shared', we just
+    # NOTE(venkatesh) if the 'visibility' is set to 'shared', we just
     # query the image members table. No union is required.
     if visibility is not None and visibility == 'shared':
         return query_member
 
-    query_image = session.query(models.Image)\
-        .filter(img_conditional_clause)
+    query_image = session.query(models.Image).filter(img_conditional_clause)
     if regular_user:
         query_image = query_image.filter(models.Image.is_public == True)
         query_image_owner = None
         if context.owner is not None:
-            query_image_owner = session.query(models.Image) \
-                .filter(models.Image.owner == context.owner) \
-                .filter(img_conditional_clause)
+            query_image_owner = session.query(models.Image).filter(
+                models.Image.owner == context.owner).filter(
+                    img_conditional_clause)
         if query_image_owner is not None:
             query = query_image.union(query_image_owner, query_member)
         else:
             query = query_image.union(query_member)
         return query
     else:
-        #Admin user
+        # Admin user
         return query_image
 
 
 def image_get_all(context, filters=None, marker=None, limit=None,
-                  sort_key='created_at', sort_dir='desc',
+                  sort_key=None, sort_dir=None,
                   member_status='accepted', is_public=None,
                   admin_as_user=False, return_tag=False):
     """
@@ -535,30 +544,40 @@ def image_get_all(context, filters=None, marker=None, limit=None,
                     filters on the image properties attribute
     :param marker: image id after which to start page
     :param limit: maximum number of images to return
-    :param sort_key: image attribute by which results should be sorted
-    :param sort_dir: direction in which results should be sorted (asc, desc)
+    :param sort_key: list of image attributes by which results should be sorted
+    :param sort_dir: directions in which results should be sorted (asc, desc)
     :param member_status: only return shared images that have this membership
                           status
     :param is_public: If true, return only public images. If false, return
                       only private and shared images.
     :param admin_as_user: For backwards compatibility. If true, then return to
                       an admin the equivalent set of images which it would see
-                      if it were a regular user
+                      if it was a regular user
     :param return_tag: To indicates whether image entry in result includes it
                        relevant tag entries. This could improve upper-layer
                        query performance, to prevent using separated calls
     """
+    sort_key = ['created_at'] if not sort_key else sort_key
+
+    default_sort_dir = 'desc'
+
+    if not sort_dir:
+        sort_dir = [default_sort_dir] * len(sort_key)
+    elif len(sort_dir) == 1:
+        default_sort_dir = sort_dir[0]
+        sort_dir *= len(sort_key)
+
     filters = filters or {}
 
     visibility = filters.pop('visibility', None)
     showing_deleted = 'changes-since' in filters or filters.get('deleted',
                                                                 False)
 
-    img_conditions, prop_conditions, tag_conditions = \
-        _make_conditions_from_filters(filters, is_public)
+    img_cond, prop_cond, tag_cond = _make_conditions_from_filters(
+        filters, is_public)
 
     query = _select_images_query(context,
-                                 img_conditions,
+                                 img_cond,
                                  admin_as_user,
                                  member_status,
                                  visibility)
@@ -569,15 +588,15 @@ def image_get_all(context, filters=None, marker=None, limit=None,
         elif visibility == 'private':
             query = query.filter(models.Image.is_public == False)
 
-    if prop_conditions:
-        for prop_condition in prop_conditions:
-            query = query.join(models.ImageProperty, aliased=True)\
-                .filter(sa_sql.and_(*prop_condition))
+    if prop_cond:
+        for prop_condition in prop_cond:
+            query = query.join(models.ImageProperty, aliased=True).filter(
+                sa_sql.and_(*prop_condition))
 
-    if tag_conditions:
-        for tag_condition in tag_conditions:
-            query = query.join(models.ImageTag, aliased=True)\
-                .filter(sa_sql.and_(*tag_condition))
+    if tag_cond:
+        for tag_condition in tag_cond:
+            query = query.join(models.ImageTag, aliased=True).filter(
+                sa_sql.and_(*tag_condition))
 
     marker_image = None
     if marker is not None:
@@ -585,23 +604,27 @@ def image_get_all(context, filters=None, marker=None, limit=None,
                                   marker,
                                   force_show_deleted=showing_deleted)
 
-    sort_keys = ['created_at', 'id']
-    sort_keys.insert(0, sort_key) if sort_key not in sort_keys else sort_keys
+    for key in ['created_at', 'id']:
+        if key not in sort_key:
+            sort_key.append(key)
+            sort_dir.append(default_sort_dir)
 
     query = _paginate_query(query, models.Image, limit,
-                            sort_keys,
+                            sort_key,
                             marker=marker_image,
-                            sort_dir=sort_dir)
+                            sort_dir=None,
+                            sort_dirs=sort_dir)
 
-    query = query.options(sa_orm.joinedload(models.Image.properties))\
-                 .options(sa_orm.joinedload(models.Image.locations))
+    query = query.options(sa_orm.joinedload(
+        models.Image.properties)).options(
+            sa_orm.joinedload(models.Image.locations))
     if return_tag:
         query = query.options(sa_orm.joinedload(models.Image.tags))
 
     images = []
     for image in query.all():
         image_dict = image.to_dict()
-        image_dict = _normalize_locations(image_dict,
+        image_dict = _normalize_locations(context, image_dict,
                                           force_show_deleted=showing_deleted)
         if return_tag:
             image_dict = _normalize_tags(image_dict)
@@ -663,6 +686,7 @@ def _update_values(image_ref, values):
 
 @retry(retry_on_exception=_retry_on_deadlock, wait_fixed=500,
        stop_max_attempt_number=50)
+@utils.no_4byte_params
 def _image_update(context, values, image_id, purge_props=False,
                   from_state=None):
     """
@@ -673,7 +697,7 @@ def _image_update(context, values, image_id, purge_props=False,
     :param image_id: If None, create the image, otherwise, find and update it
     """
 
-    #NOTE(jbresnah) values is altered in this so a copy is needed
+    # NOTE(jbresnah) values is altered in this so a copy is needed
     values = values.copy()
 
     session = get_session()
@@ -716,7 +740,7 @@ def _image_update(context, values, image_id, purge_props=False,
         if image_id:
             # Don't drop created_at if we're passing it in...
             _drop_protected_attrs(models.Image, values)
-            #NOTE(iccha-sethi): updated_at must be explicitly set in case
+            # NOTE(iccha-sethi): updated_at must be explicitly set in case
             #                   only ImageProperty table was modifited
             values['updated_at'] = timeutils.utcnow()
 
@@ -773,6 +797,7 @@ def _image_update(context, values, image_id, purge_props=False,
     return image_get(context, image_ref.id)
 
 
+@utils.no_4byte_params
 def image_location_add(context, image_id, location, session=None):
     deleted = location['status'] in ('deleted', 'pending_delete')
     delete_time = timeutils.utcnow() if deleted else None
@@ -786,6 +811,7 @@ def image_location_add(context, image_id, location, session=None):
     location_ref.save(session=session)
 
 
+@utils.no_4byte_params
 def image_location_update(context, image_id, location, session=None):
     loc_id = location.get('id')
     if loc_id is None:
@@ -794,10 +820,8 @@ def image_location_update(context, image_id, location, session=None):
 
     try:
         session = session or get_session()
-        location_ref = session.query(models.ImageLocation)\
-            .filter_by(id=loc_id)\
-            .filter_by(image_id=image_id)\
-            .one()
+        location_ref = session.query(models.ImageLocation).filter_by(
+            id=loc_id).filter_by(image_id=image_id).one()
 
         deleted = location['status'] in ('deleted', 'pending_delete')
         updated_time = timeutils.utcnow()
@@ -826,10 +850,8 @@ def image_location_delete(context, image_id, location_id, status,
 
     try:
         session = session or get_session()
-        location_ref = session.query(models.ImageLocation)\
-            .filter_by(id=location_id)\
-            .filter_by(image_id=image_id)\
-            .one()
+        location_ref = session.query(models.ImageLocation).filter_by(
+            id=location_id).filter_by(image_id=image_id).one()
 
         delete_time = delete_time or timeutils.utcnow()
 
@@ -848,12 +870,12 @@ def image_location_delete(context, image_id, location_id, status,
 def _image_locations_set(context, image_id, locations, session=None):
     # NOTE(zhiyan): 1. Remove records from DB for deleted locations
     session = session or get_session()
-    query = session.query(models.ImageLocation) \
-        .filter_by(image_id=image_id) \
-        .filter_by(deleted=False) \
-        .filter(~models.ImageLocation.id.in_([loc['id']
-                                              for loc in locations
-                                              if loc.get('id')]))
+    query = session.query(models.ImageLocation).filter_by(
+        image_id=image_id).filter_by(
+            deleted=False).filter(~models.ImageLocation.id.in_(
+                [loc['id']
+                 for loc in locations
+                 if loc.get('id')]))
     for loc_id in [loc_ref.id for loc_ref in query.all()]:
         image_location_delete(context, image_id, loc_id, 'deleted',
                               session=session)
@@ -870,16 +892,15 @@ def _image_locations_delete_all(context, image_id,
                                 delete_time=None, session=None):
     """Delete all image locations for given image"""
     session = session or get_session()
-    location_refs = session.query(models.ImageLocation) \
-        .filter_by(image_id=image_id) \
-        .filter_by(deleted=False) \
-        .all()
+    location_refs = session.query(models.ImageLocation).filter_by(
+        image_id=image_id).filter_by(deleted=False).all()
 
     for loc_id in [loc_ref.id for loc_ref in location_refs]:
         image_location_delete(context, image_id, loc_id, 'deleted',
                               delete_time=delete_time, session=session)
 
 
+@utils.no_4byte_params
 def _set_properties_for_image(context, image_ref, properties,
                               purge_props=False, session=None):
     """
@@ -935,9 +956,8 @@ def _image_child_entry_delete_all(child_model_cls, image_id, delete_time=None,
     """
     session = session or get_session()
 
-    query = session.query(child_model_cls) \
-        .filter_by(image_id=image_id) \
-        .filter_by(deleted=False)
+    query = session.query(child_model_cls).filter_by(
+        image_id=image_id).filter_by(deleted=False)
 
     delete_time = delete_time or timeutils.utcnow()
 
@@ -1102,21 +1122,8 @@ def image_member_count(context, image_id):
     return query.count()
 
 
-# pylint: disable-msg=C0111
-def _can_show_deleted(context):
-    """
-    Calculates whether to include deleted objects based on context.
-    Currently just looks for a flag called deleted in the context dict.
-    """
-    if hasattr(context, 'show_deleted'):
-        return context.show_deleted
-    if not hasattr(context, 'get'):
-        return False
-    return context.get('deleted', False)
-
-
 def image_tag_set_all(context, image_id, tags):
-    #NOTE(kragniz): tag ordering should match exactly what was provided, so a
+    # NOTE(kragniz): tag ordering should match exactly what was provided, so a
     # subsequent call to image_tag_get_all returns them in the correct order
 
     session = get_session()
@@ -1133,6 +1140,7 @@ def image_tag_set_all(context, image_id, tags):
             image_tag_delete(context, image_id, tag, session)
 
 
+@utils.no_4byte_params
 def image_tag_create(context, image_id, value, session=None):
     """Create an image tag."""
     session = session or get_session()
@@ -1145,10 +1153,9 @@ def image_tag_delete(context, image_id, value, session=None):
     """Delete an image tag."""
     _check_image_id(image_id)
     session = session or get_session()
-    query = session.query(models.ImageTag)\
-                   .filter_by(image_id=image_id)\
-                   .filter_by(value=value)\
-                   .filter_by(deleted=False)
+    query = session.query(models.ImageTag).filter_by(
+        image_id=image_id).filter_by(
+            value=value).filter_by(deleted=False)
     try:
         tag_ref = query.one()
     except sa_orm.exc.NoResultFound:
@@ -1170,10 +1177,8 @@ def image_tag_get_all(context, image_id, session=None):
     """Get a list of tags for a specific image."""
     _check_image_id(image_id)
     session = session or get_session()
-    tags = session.query(models.ImageTag.value)\
-                  .filter_by(image_id=image_id)\
-                  .filter_by(deleted=False)\
-                  .all()
+    tags = session.query(models.ImageTag.value).filter_by(
+        image_id=image_id).filter_by(deleted=False).all()
     return [tag[0] for tag in tags]
 
 
@@ -1321,8 +1326,7 @@ def task_get_all(context, filters=None, marker=None, limit=None,
     session = get_session()
     query = session.query(models.Task)
 
-    if not (context.is_admin or admin_as_user == True) and \
-            context.owner is not None:
+    if not (context.is_admin or admin_as_user) and context.owner is not None:
         query = query.filter(models.Task.owner == context.owner)
 
     showing_deleted = False
@@ -1386,7 +1390,7 @@ def _task_get(context, task_id, session=None, force_show_deleted=False):
         sa_orm.joinedload(models.Task.info)
     ).filter_by(id=task_id)
 
-    if not force_show_deleted and not _can_show_deleted(context):
+    if not force_show_deleted and not context.can_see_deleted:
         query = query.filter_by(deleted=False)
     try:
         task_ref = query.one()
@@ -1631,3 +1635,121 @@ def metadef_resource_type_association_get_all_by_namespace(
     session = session or get_session()
     return metadef_association_api.\
         get_all_by_namespace(context, namespace_name, session)
+
+
+def metadef_tag_get_all(
+        context, namespace_name, filters=None, marker=None, limit=None,
+        sort_key=None, sort_dir=None, session=None):
+    """Get metadata-schema tags or raise if none exist."""
+    session = session or get_session()
+    return metadef_tag_api.get_all(
+        context, namespace_name, session,
+        filters, marker, limit, sort_key, sort_dir)
+
+
+def metadef_tag_get(context, namespace_name, name, session=None):
+    """Get a metadata-schema tag or raise if it does not exist."""
+    session = session or get_session()
+    return metadef_tag_api.get(
+        context, namespace_name, name, session)
+
+
+def metadef_tag_create(context, namespace_name, tag_dict,
+                       session=None):
+    """Create a metadata-schema tag or raise if it already exists."""
+    session = session or get_session()
+    return metadef_tag_api.create(
+        context, namespace_name, tag_dict, session)
+
+
+def metadef_tag_create_tags(context, namespace_name, tag_list,
+                            session=None):
+    """Create a metadata-schema tag or raise if it already exists."""
+    session = get_session()
+    return metadef_tag_api.create_tags(
+        context, namespace_name, tag_list, session)
+
+
+def metadef_tag_update(context, namespace_name, id, tag_dict,
+                       session=None):
+    """Update an tag or raise if it does not exist or not visible."""
+    session = session or get_session()
+    return metadef_tag_api.update(
+        context, namespace_name, id, tag_dict, session)
+
+
+def metadef_tag_delete(context, namespace_name, name,
+                       session=None):
+    """Delete an tag or raise if namespace or tag doesn't exist."""
+    session = session or get_session()
+    return metadef_tag_api.delete(
+        context, namespace_name, name, session)
+
+
+def metadef_tag_delete_namespace_content(
+        context, namespace_name, session=None):
+    """Delete an tag or raise if namespace or tag doesn't exist."""
+    session = session or get_session()
+    return metadef_tag_api.delete_by_namespace_name(
+        context, namespace_name, session)
+
+
+def metadef_tag_count(context, namespace_name, session=None):
+    """Get count of tags for a namespace, raise if ns doesn't exist."""
+    session = session or get_session()
+    return metadef_tag_api.count(context, namespace_name, session)
+
+
+def artifact_create(context, values, type_name,
+                    type_version=None, session=None):
+    session = session or get_session()
+    artifact = artifacts.create(context, values, session, type_name,
+                                type_version)
+    return artifact
+
+
+def artifact_delete(context, artifact_id, type_name,
+                    type_version=None, session=None):
+    session = session or get_session()
+    artifact = artifacts.delete(context, artifact_id, session, type_name,
+                                type_version)
+    return artifact
+
+
+def artifact_update(context, values, artifact_id, type_name,
+                    type_version=None, session=None):
+    session = session or get_session()
+    artifact = artifacts.update(context, values, artifact_id, session,
+                                type_name, type_version)
+    return artifact
+
+
+def artifact_get(context, artifact_id,
+                 type_name=None,
+                 type_version=None,
+                 show_level=ga.Showlevel.BASIC,
+                 session=None):
+    session = session or get_session()
+    return artifacts.get(context, artifact_id, session, type_name,
+                         type_version, show_level)
+
+
+def artifact_publish(context,
+                     artifact_id,
+                     type_name,
+                     type_version=None,
+                     session=None):
+    session = session or get_session()
+    return artifacts.publish(context,
+                             artifact_id,
+                             session,
+                             type_name,
+                             type_version)
+
+
+def artifact_get_all(context, marker=None, limit=None, sort_keys=None,
+                     sort_dirs=None, filters=None,
+                     show_level=ga.Showlevel.NONE, session=None):
+    session = session or get_session()
+    return artifacts.get_all(context, session, marker, limit, sort_keys,
+                             sort_dirs, filters, show_level)
